from viggocore.database import db
from viggocore.common.subsystem import entity


class UficmsSugestao(entity.Entity, db.Model):

    attributes = ['uf_origem', 'uf_destino', 'aliquota']
    attributes += entity.Entity.attributes

    uf_origem = db.Column(db.CHAR(2), nullable=False)
    uf_destino = db.Column(db.CHAR(2), nullable=False)
    aliquota = db.Column(db.Numeric(10), nullable=False)

    def __init__(self, id, uf_origem, uf_destino, aliquota,
                 active=True, created_at=None, created_by=None,
                 updated_at=None, updated_by=None, tag=None):
        super().__init__(id, active, created_at, created_by,
                         updated_at, updated_by, tag)
        self.uf_origem = uf_origem
        self.uf_destino = uf_destino
        self.aliquota = aliquota

    @classmethod
    def individual(cls):
        return 'uficms_sugestao'

    @classmethod
    def collection(cls):
        return 'uficms_sugestoes'



from viggocore.common import subsystem
from viggofiscal.subsystem.parametrizacao.regra_fiscal import resource, manager
from viggocore.common import controller

subsystem = subsystem.Subsystem(resource=resource.RegraFiscal,
                                manager=manager.Manager,
                                controller=controller.CommonController)



from sqlalchemy import UniqueConstraint
from viggocore.database import db
from viggocore.common.subsystem import entity


class Uficms(entity.Entity, db.Model):

    attributes = ['sigla', 'aliq_icms_proprio', 'aliq_icms_st', 'aliq_fcp',
                  'aliq_fcp_st']
    attributes += entity.Entity.attributes

    domain_org_id = db.Column(
        db.CHAR(32), db.ForeignKey('domain_org.id'), nullable=False)

    sigla = db.Column(db.CHAR(2), nullable=False)
    aliq_icms_proprio = db.Column(db.Numeric(5, 2), nullable=False)
    aliq_icms_st = db.Column(db.Numeric(5, 2), nullable=False)
    aliq_fcp = db.Column(db.Numeric(5, 2), nullable=False)
    aliq_fcp_st = db.Column(db.Numeric(5, 2), nullable=False)

    __table_args__ = (
        UniqueConstraint('domain_org_id', 'sigla',
                         name='uficms_domain_org_id_sigla_uk'),)

    def __init__(self, id, domain_org_id, sigla, aliq_icms_proprio,
                 aliq_icms_st, aliq_fcp, aliq_fcp_st,
                 active=True, created_at=None, created_by=None,
                 updated_at=None, updated_by=None, tag=None):
        super().__init__(id, active, created_at, created_by,
                         updated_at, updated_by, tag)
        self.domain_org_id = domain_org_id
        self.sigla = sigla
        self.aliq_icms_proprio = aliq_icms_proprio
        self.aliq_icms_st = aliq_icms_st
        self.aliq_fcp = aliq_fcp
        self.aliq_fcp_st = aliq_fcp_st



from viggocore.database import db
from viggocore.common.subsystem import entity


class Origem(entity.Entity, db.Model):

    attributes = ['origem', 'descricao']
    attributes += entity.Entity.attributes

    origem = db.Column(db.String(2), nullable=False, unique=True)
    descricao = db.Column(db.String(100), nullable=False)

    def __init__(self, id, origem, descricao,
                 active=True, created_at=None, created_by=None,
                 updated_at=None, updated_by=None, tag=None):
        super().__init__(id, active, created_at, created_by,
                         updated_at, updated_by, tag)
        self.origem = origem
        self.descricao = descricao

    @classmethod
    def collection(cls):
        return 'origens'



from viggocore.common.subsystem import operation
from viggocore.common.subsystem.pagination import Pagination
from viggocore.common import manager

from viggofiscal.subsystem.parametrizacao.cfop.resource import Cfop
from viggofiscal.subsystem.parametrizacao.regra_fiscal.resource \
    import RegraFiscalModelo


class List(operation.List):

    def do(self, session, **kwargs):

        query = session.query(Cfop)

        modelo = kwargs.get('modelo', None)
        if modelo and int(modelo) is RegraFiscalModelo.NFCE.value:
            nfce_cfop_list = ['5101', '5102', '5103', '5104', '5115',
                              '5405', '5656', '5667']
            query = query.filter(Cfop.cfop.in_(nfce_cfop_list))

        query = self.manager.apply_filters(query, Cfop, **kwargs)
        query = query.distinct()

        total_rows = None
        if self.manager.with_pagination(**kwargs):
            total_rows = query.count()

        pagination = Pagination.get_pagination(Cfop, **kwargs)
        if pagination.order_by is not None:
            pagination.adjust_order_by(Cfop)
        query = self.driver.apply_pagination(query, pagination)
        result = query.all()

        return (result, total_rows)


class Manager(manager.CommonManager):

    def __init__(self, driver):
        super(Manager, self).__init__(driver)
        self.list = List(self)



from viggocore.database import db
from viggocore.common.subsystem import entity
from sqlalchemy import UniqueConstraint


class TipoOperacao(entity.Entity, db.Model):

    CODIGO_SEQUENCE = 'tipo_operacao_codigo_sq'

    attributes = ['domain_org_id', 'codigo', 'descricao']
    attributes += entity.Entity.attributes

    domain_org_id = db.Column(
        db.CHAR(32), db.ForeignKey('domain_org.id'), nullable=False)

    codigo = db.Column(db.Numeric(10), db.Sequence(CODIGO_SEQUENCE),
                       nullable=False)
    descricao = db.Column(db.String(100), nullable=False)

    __table_args__ = (
        UniqueConstraint(
            'domain_org_id', 'codigo',
            name='tipo_operacao_domain_org_id_codigo_uk'),)

    def __init__(self, id, domain_org_id, codigo, descricao,
                 active=True, created_at=None, created_by=None,
                 updated_at=None, updated_by=None, tag=None):
        super().__init__(id, active, created_at, created_by,
                         updated_at, updated_by, tag)
        self.domain_org_id = domain_org_id
        self.codigo = codigo
        self.descricao = descricao

    @classmethod
    def individual(cls):
        return 'tipo_operacao'



import flask
from viggocore.common import exception, utils
from viggocore.common.subsystem import controller


class Controller(controller.Controller):

    def update(self, id):
        data = flask.request.get_json()
        data.pop('id', None)

        try:
            entity = self.manager.update(id=id, **data)
        except exception.ViggoCoreException as exc:
            return flask.Response(response=exc.message,
                                  status=exc.status)

        response = {self.resource_wrap: entity.to_dict()}

        return flask.Response(response=utils.to_json(response),
                              status=200,
                              mimetype="application/json")



from viggocore.common import subsystem
from viggofiscal.subsystem.parametrizacao.ncm_ibpt_erro \
    import resource, controller, manager, router

subsystem = subsystem.Subsystem(resource=resource.NcmIbptErro,
                                controller=controller.Controller,
                                manager=manager.Manager,
                                router=router.Router)



import csv

import flask
from viggocore.common import exception, utils
from viggocore.common.subsystem import controller
from datetime import datetime
from sqlalchemy import exc as sqlexc


class Controller(controller.Controller):

    def convert_string_to_date(self, date: str):
        return datetime.strptime(date, '%d/%m/%Y')

    def get_ex(self, ex: str):
        if ex == '':
            return None
        else:
            return int(ex)

    def limpar_ncmibpt_e_os_ncmibpt_erros_por_sigla(self, sigla_uf):
        response = self.manager.limpar_por_sigla(**{'sigla': sigla_uf})
        return response

    def make_ncm_ibpt_erro_dict(self, linha, sigla_uf, filename, msg_erro):
        erro_dict = {
            "ncm": linha['ncm'],
            "uf": sigla_uf,
            "chave": linha['chave'],
            "versao": linha['versao'],
            "extipi":  linha['extipi'],
            "tipo": linha['tipo'],
            "filename": filename,
            "msg_erro": msg_erro,
            "erro_em": datetime.now()
        }
        return erro_dict

    def make_ncmibpt_dict(self, ncmibpt_file_line, sigla_uf, user_id):
        ncmibpt_dict = {
            'ncm': ncmibpt_file_line['codigo'],
            'uf': sigla_uf,
            'chave': ncmibpt_file_line['chave'],
            'versao': ncmibpt_file_line['versao'],
            'descricao': ncmibpt_file_line['descricao'],
            'aliq_nacional': float(ncmibpt_file_line['nacionalfederal']),
            'aliq_importacao': float(ncmibpt_file_line['importadosfederal']),
            'aliq_estadual': float(ncmibpt_file_line['estadual']),
            'aliq_municipal': float(ncmibpt_file_line['municipal']),
            'inicio_vigencia': self.convert_string_to_date(
                ncmibpt_file_line['vigenciainicio']),
            'fim_vigencia': self.convert_string_to_date(
                ncmibpt_file_line['vigenciafim']),
            'extipi': self.get_ex(ncmibpt_file_line['ex']),
            'tipo': self.get_ex(ncmibpt_file_line['tipo']),
            'created_by': user_id
        }
        return ncmibpt_dict

    def get_user_id(self):
        user_id = None
        if flask.has_request_context():
            token_id = flask.request.headers.get('token')
            if token_id is not None:
                self.token = self.manager.api.tokens().get(id=token_id)
                user_id = self.token.user_id
        return user_id

    def get_linhas_dict(self, ncmibpt_file_lines, sigla_uf, user_id):
        linhas = []
        for ncmibpt_file_line in ncmibpt_file_lines:
            linhas.append(
                self.make_ncmibpt_dict(ncmibpt_file_line, sigla_uf, user_id))
        return linhas

    def get_erro_in_db(self, erro):
        filter_dict = {
            "ncm": erro['ncm'],
            "uf": erro['uf'],
            "chave": erro['chave'],
            "versao": erro['versao'],
            "extipi": erro['extipi']
        }
        erro = self.manager.api.ncm_ibpt_erros().get_erro_existe(
            **filter_dict)
        return erro

    def cadastrar_ncm_ibpt_erro(self, erro, user_id) -> None:
        exists = self.get_erro_in_db(erro)
        if exists is None:
            erro['created_by'] = user_id
            self.manager.api.ncm_ibpt_erros().create(**erro)
        else:
            self.manager.api.ncm_ibpt_erros().update(
                id=exists.id,
                **{'updated_at': datetime.now(), 'updated_by': user_id})

    def cadastrar_por_arquivo(self):
        file = flask.request.files.get('file', None)
        sigla_uf = flask.request.form.get('sigla_uf', None)
        erros = []
        linhas = []
        tipo = file.mimetype.split('/')[-1].upper()

        if tipo == 'CSV':
            filename = file.filename
            file_content = file.read()
            ncmibpt_file_lines = csv.DictReader(
                file_content.decode('unicode_escape').splitlines(),
                delimiter=';')
            if len(ncmibpt_file_lines.fieldnames) == 1:
                msg = ("O delimitador de coluna usado no código foi o ';', " +
                       "mas apenas uma coluna foi gerada então verifique o " +
                       "delimitador de colunas no arquivo.")
                return flask.Response(response=msg,
                                      mimetype="application/text",
                                      status=400)
            user_id = self.get_user_id()

            linhas = self.get_linhas_dict(
                ncmibpt_file_lines, sigla_uf, user_id)

            line_count = 0
            qtd_sucesso = 0
            self.limpar_ncmibpt_e_os_ncmibpt_erros_por_sigla(sigla_uf=sigla_uf)
            for linha in linhas:
                if line_count != 0:
                    if linha['tipo'] == 0:
                        try:
                            self.manager.driver.transaction_manager.count = 0
                            self.manager.create(**linha)
                            qtd_sucesso += 1
                        except exception.ViggoCoreException as exc:
                            erro_dict = self.make_ncm_ibpt_erro_dict(
                                linha,
                                sigla_uf,
                                filename,
                                exc.message)
                            erros.append(erro_dict)
                            self.cadastrar_ncm_ibpt_erro(erro_dict, user_id)
                        except sqlexc.DataError as exc:
                            erro_dict = self.make_ncm_ibpt_erro_dict(
                                linha,
                                sigla_uf,
                                filename,
                                str(exc))
                            erros.append(erro_dict)
                            self.cadastrar_ncm_ibpt_erro(erro_dict, user_id)
                        except AttributeError as exc:
                            erro_dict = self.make_ncm_ibpt_erro_dict(
                                linha,
                                sigla_uf,
                                filename,
                                str(exc))
                            erros.append(erro_dict)
                            self.cadastrar_ncm_ibpt_erro(erro_dict, user_id)
                        except Exception as exc:
                            erro_dict = self.make_ncm_ibpt_erro_dict(
                                linha,
                                sigla_uf,
                                filename,
                                exc.message)
                            erros.append(erro_dict)
                            self.cadastrar_ncm_ibpt_erro(erro_dict, user_id)
                else:
                    line_count += 1
        else:
            msg = 'A aplicação só permite arquivos .csv'
            return flask.Response(response=msg,
                                  status=400)

        response = {
            'filename': filename,
            'sigla_uf': sigla_uf,
            'qtd_sucesso': qtd_sucesso,
            'qtd_erros': len(erros),
            'erros': erros}

        return flask.Response(response=utils.to_json(response),
                              status=201,
                              mimetype="application/json")




from viggocore.common.subsystem import router


class Router(router.Router):

    def __init__(self, collection, routes=[]):
        super().__init__(collection, routes)

    @property
    def routes(self):
        return super().routes + [
            {
                'action': 'Ajustar saldo inicial do portador',
                'method': 'PUT',
                'url': self.resource_url + '/ajuste_saldo',
                'callback': 'ajuste_saldo'
            }
        ]



import flask
from viggocore.common import exception, utils, controller


class Controller(controller.CommonController):

    def __init__(self, manager, resource_wrap, collection_wrap):
        super(Controller, self).__init__(
            manager, resource_wrap, collection_wrap)

    def analisar(self):
        data = flask.request.get_json()
        try:
            tuplas = self.manager.analisar(**data)
        except exception.ViggoCoreException as exc:
            return flask.Response(response=exc.message,
                                  status=exc.status)

        collection = []
        if len(tuplas) > 0:
            permissao = False
        else:
            permissao = True
        for tupla in tuplas:
            tupla_dict = {
                'user_name': tupla[0],
                'terminal_descricao': tupla[1]}
            collection.append(tupla_dict)

        response = {'permissao': permissao, 'operadores_em_uso': collection}

        return flask.Response(response=utils.to_json(response),
                              status=200,
                              mimetype="application/json")



import math
from decimal import Decimal


def to_decimal(num):
    try:
        return Decimal(str(num))
    except Exception:
        return Decimal('0.0')


def to_decimal_n(num, n):
    try:
        num_str = '{:.' + str(n) + 'f}'
        num_str = num_str.format(num)
        return Decimal(num_str)
    except Exception:
        return Decimal('0.0')


def is_decimal(num):
    response = False
    if (to_decimal(num) - int(num)) > to_decimal('0.0'):
        response = True
    return response


def normalize_number_to_print(valor):
    valor_decimal = to_decimal(valor)
    if valor_decimal - int(valor_decimal) != 0:
        return '{:.2f}'.format(valor_decimal)
    else:
        return int(valor_decimal)


# verifica se um número é par
def se_e_par(n):
    n = int(n)
    if (n % 2) == 0:
        return True
    else:
        return False


# função criada para arredondar um valor usando a norma ABNT 5891/77
# onde valor=o valor a ser arredondado
# n=limitador da quantidade de casas
def round_abnt_video(valor, n: int):
    try:
        vl_str = str(valor)
        if '.' in vl_str:
            inteira, decimal = str(valor).split('.')
        else:
            inteira = str(valor)
            decimal = '00'
    except Exception:
        print('Erro na conversão do valor!')

    decimal = (decimal + ('0' * 10))
    if len(decimal) > (n+2):
        prox = int(decimal[n])
        pos_prox = int(decimal[n+1])
        if ((prox >= 5 and pos_prox != 0) or
           (prox == 5 and pos_prox == 0 and se_e_par(decimal[n-1]) is False)):
            aux = int(decimal[n-1]) + 1
            decimal = decimal[:n-1] + str(aux) + decimal[n:]
    decimal = decimal[:n]
    return Decimal(f'{inteira}.{decimal}')


def preencher_decimal(valor):
    try:
        vl_str = str(valor)
        if '.' not in vl_str:
            vl_str = vl_str + '.0'
    except Exception:
        print('Erro na conversão do valor!')

    return vl_str


def float_to_string(n: float):
    if 'e-' in str(n):
        values = str(n).split('e-')
        a = values[0].replace('.', '')
        b = int(values[1])
        return '0.' + ('0' * (b-1)) + a
    else:
        return str(n)


# função criada para arredondar um valor usando a norma ABNT do delphi onde:
# valor=0 valor a ser arredondado
# n=limitador da quantidade de casas
def round_abnt(valor, n: int):
    valor = float(valor)
    negativo = valor < 0

    delta = 0.00001
    pow_aux = pow(10, n)
    pow_value = abs(valor) / 10
    int_value = math.trunc(pow_value)
    frac_value = float('0.' + float_to_string(pow_value).split('.')[1])
    pow_value = (
        (frac_value * 10 * pow_aux / pow(10.0, -9)) + 0.5) * pow(10.0, -9)
    int_calc = math.trunc(pow_value)
    frac_calc = float('0.' + float_to_string(pow_value).split('.')[1])
    frac_calc = math.trunc(frac_calc * 100)

    if (frac_calc > 50):
        int_calc += 1
    elif (frac_calc == 50):
        valor_temp = int_calc / 10
        last_number = round(
            (float('0.' + float_to_string(valor_temp).split('.')[1])) * 10)
        if (last_number % 2) == 1:
            int_calc += 1
        else:
            valor_temp = pow_value * 10
            rest_part = float('0.' + float_to_string(valor_temp).split('.')[1])
            if rest_part > delta:
                int_calc += 1

    resultado = ((int_value * 10) + (int_calc / pow_aux))
    if negativo:
        resultado *= -1
    return resultado



from viggocore.common import subsystem
from viggofiscal.subsystem.parametrizacao.forma_pagamento_sefaz \
    import resource, router

subsystem = subsystem.Subsystem(resource=resource.FormaPagamentoSefaz,
                                router=router.Router)



import flask
from viggocore.common import exception, utils, controller


class Controller(controller.CommonController):

    def __init__(self, manager, resource_wrap, collection_wrap):
        super(Controller, self).__init__(
            manager, resource_wrap, collection_wrap)



from viggocore.common import manager
from viggocore.common.subsystem import operation
from viggocore.common.subsystem.pagination import Pagination

from viggofiscal.subsystem.parametrizacao.uficms_sugestao.resource import \
    UficmsSugestao


class List(operation.List):

    def do(self, session, **kwargs):
        query = session.query(UficmsSugestao)
        query = self.manager.apply_filters(query, UficmsSugestao, **kwargs)

        dict_compare = {}
        query = self.manager.apply_filters_includes(
            query, dict_compare, **kwargs)

        total_rows = None
        if self.manager.with_pagination(**kwargs):
            total_rows = query.count()

        pagination = Pagination.get_pagination(UficmsSugestao, **kwargs)
        if pagination.order_by is not None:
            pagination.adjust_order_by(UficmsSugestao)
        query = self.driver.apply_pagination(query, pagination)
        result = query.all()

        return (result, total_rows)


class Manager(manager.CommonManager):

    def __init__(self, driver):
        super(Manager, self).__init__(driver)
        self.list = List(self)



from viggocore.common import subsystem
from viggofiscal.subsystem.parametrizacao.serial_fiscal \
    import resource, manager, driver, controller, router

subsystem = subsystem.Subsystem(resource=resource.SerieFiscal,
                                manager=manager.Manager,
                                driver=driver.Driver,
                                controller=controller.Controller,
                                router=router.Router)



from viggofiscal.subsystem.calculo_fiscal.icms.base_reduzida_icms_proprio \
    import BaseReduzidaIcmsProprio
from viggofiscal.subsystem.calculo_fiscal.icms.base_icms_proprio \
    import BaseIcmsProprio
from viggofiscal.subsystem.calculo_fiscal.utils import round_abnt


class Icms101():

    def __init__(self, valor_produto: float, valor_frete: float,
                 valor_seguro: float, despesas_acessorias: float,
                 valor_desconto: float, percentual_credito_sn: float,
                 percentual_reducao: float = 0.0):
        self.valor_produto = float(valor_produto)
        self.valor_frete = float(valor_frete)
        self.valor_seguro = float(valor_seguro)
        self.despesas_acessorias = float(despesas_acessorias)
        self.valor_desconto = float(valor_desconto)
        self.percentual_credito_sn = float(percentual_credito_sn)
        self.percentual_reducao = float(percentual_reducao)

    def base_icms_proprio(self):
        if self.percentual_reducao == 0:
            self.bc_icms_proprio = BaseIcmsProprio(
                self.valor_produto, self.valor_frete, self.valor_seguro,
                self.despesas_acessorias, self.valor_desconto)
            return self.bc_icms_proprio.calcular_base_icms_proprio()
        else:
            self.bc_reduzida_icms_proprio = BaseReduzidaIcmsProprio(
                self.valor_produto, self.valor_frete, self.valor_seguro,
                self.despesas_acessorias, self.valor_desconto,
                self.percentual_reducao)
            return self.bc_reduzida_icms_proprio.\
                calcular_base_reduzida_icms_proprio()

    def valor_credito_sn(self):
        valor_credito_sn = (
            self.base_icms_proprio() * (self.percentual_credito_sn / 100))
        return round_abnt(valor_credito_sn, 2)




import flask

from viggocore.common import controller, exception, utils


class Controller(controller.CommonController):

    def __init__(self, manager, resource_wrap, collection_wrap):
        super(Controller, self).__init__(
            manager, resource_wrap, collection_wrap)

    def ajuste_saldo(self, id):
        data = flask.request.get_json()

        try:
            if data:
                id = data.pop('id', id)
                entity = self.manager.ajusta_saldo_inicial(id=id, **data)
            else:
                entity = self.manager.ajusta_saldo_inicial(id=id)
        except exception.ViggoCoreException as exc:
            return flask.Response(response=exc.message,
                                  status=exc.status)

        response = {self.resource_wrap: entity.to_dict()}

        return flask.Response(response=utils.to_json(response),
                              status=201,
                              mimetype="application/json")



from viggocore.common.subsystem import operation
from viggocore.common.subsystem.pagination import Pagination
from viggocore.common import manager, utils

from viggofiscal.subsystem.financeiro.portador.resource import Portador
from viggofiscal.subsystem.parametrizacao.terminal.resource import Terminal


class Create(operation.Create):

    def pre(self, session, **kwargs):
        saldo_inicial = kwargs.get('saldo_inicial', None)
        kwargs['saldo'] = saldo_inicial

        return super().pre(session, **kwargs)


class List(operation.List):

    def do(self, session, **kwargs):
        # flag criada para listar os portadores ligados a terminais ou não
        # True = lista todos, COM ou SEM ligação com aos Terminais
        # False = lista os que NÃO ESTÃO ligados aos Terminais
        tem_terminal = kwargs.pop('tem_terminal', False)

        query = session.query(Portador)\
            .join(Terminal, Terminal.portador_id == Portador.id,
                  isouter=True)

        query = self.manager.apply_filters(query, Portador, **kwargs)
        dict_compare = {"terminal.": Terminal}
        query = self.manager.apply_filters_includes(
            query, dict_compare, **kwargs)

        if tem_terminal is False:
            query = query.filter(Terminal.id == None)  # noqa
        query = query.distinct()

        # filtro adicionado para evitar puxar o mesmo portador tanto na
        # origem quanto no destino ao realizar tranferência financeira
        if 'portador_id' in kwargs.keys():
            portador_id = kwargs.pop('portador_id', None)
            query = query.filter(Portador.id != portador_id)

        total_rows = None
        if self.manager.with_pagination(**kwargs):
            total_rows = query.count()

        pagination = Pagination.get_pagination(Portador, **kwargs)
        if pagination.order_by is not None:
            pagination.adjust_order_by(Portador)
        query = self.driver.apply_pagination(query, pagination)
        result = query.all()

        return (result, total_rows)


class AjustaSaldoInicial(operation.Update):

    def do(self, session, **kwargs):
        valor = kwargs.pop('valor', None)
        valor = utils.to_decimal(valor)
        kwargs['saldo_inicial'] = self.entity.saldo_inicial + valor
        kwargs['saldo'] = self.entity.saldo + valor

        return super().do(session, **kwargs)


class Manager(manager.CommonManager):

    def __init__(self, driver):
        super(Manager, self).__init__(driver)
        self.create = Create(self)
        self.list = List(self)
        self.ajusta_saldo_inicial = AjustaSaldoInicial(self)



import flask
from viggocore.common import exception, utils, controller


class Controller(controller.CommonController):

    def __init__(self, manager, resource_wrap, collection_wrap):
        super(Controller, self).__init__(
            manager, resource_wrap, collection_wrap)

    def get_disponiveis(self):
        filters = self._filters_parse()
        filters = self._filters_cleanup(filters)
        try:
            filters = self._parse_list_options(filters)
            entities, total_rows = self.manager.\
                get_disponiveis(**filters)

            page = filters.get('page', None)
            page_size = filters.get('page_size', None)
        except exception.ViggoCoreException as exc:
            return flask.Response(response=exc.message,
                                  status=exc.status)

        collection = self._entities_to_dict(
            entities, self._get_include_dicts_vex(filters))

        response = {self.collection_wrap: collection}

        if total_rows is not None:
            response.update({'pagination': {'page': int(page),
                                            'page_size': int(page_size),
                                            'total': total_rows}})

        return flask.Response(response=utils.to_json(response),
                              status=200,
                              mimetype="application/json")



from viggocore.common.subsystem import operation, manager
from viggocore.common import exception


class Create(operation.Create):

    def pre(self, session, **kwargs):
        csc = kwargs.get('csc', None)
        if csc is None:
            raise exception.BadRequest('csc é obrigatório')
        elif len(csc) < 16 or len(csc) > 36:
            raise exception.BadRequest(
                'csc deve conter entre 16 e 36 caracteres')

        return super().pre(session, **kwargs)


class Update(operation.Update):

    def pre(self, session, **kwargs):
        csc = kwargs.get('csc', None)
        if csc is not None and (len(csc) < 16 or len(csc) > 36):
            raise exception.BadRequest(
                'csc deve conter entre 16 e 36 caracteres')

        return super().pre(session, **kwargs)


class Manager(manager.Manager):

    def __init__(self, driver):
        super(Manager, self).__init__(driver)
        self.create = Create(self)
        self.update = Update(self)



from enum import Enum

import sqlalchemy
from sqlalchemy import orm
from viggocore.common.subsystem import entity
from viggocore.database import db


class CentroResultado(entity.Entity, db.Model):

    attributes = ['domain_id', 'centro_resultado_id', 'descricao']
    attributes += entity.Entity.attributes

    domain_id = db.Column(
        db.CHAR(32), db.ForeignKey('domain.id'), nullable=False)

    centro_resultado_id = db.Column(
        db.CHAR(32), db.ForeignKey('centro_resultado.id'), nullable=True)

    domain = orm.relationship(
        'Domain', backref=orm.backref('centro_resultado_domain'))

    centro_resultado = orm.relationship(
        'CentroResultado', remote_side='CentroResultado.id',
        backref=orm.backref('centro_resultado_centro_resultado'))

    descricao = db.Column(db.String(255), nullable=False)

    def __init__(self, id, domain_id, descricao,
                 centro_resultado_id=None,
                 active=True, created_at=None, created_by=None,
                 updated_at=None, updated_by=None, tag=None):
        super().__init__(id, active, created_at, created_by,
                         updated_at, updated_by, tag)
        self.domain_id = domain_id
        self.descricao = descricao
        self.centro_resultado_id = centro_resultado_id

    def is_stable(self):
        return self.centro_resultado_id != self.id and super().is_stable()

    @classmethod
    def individual(cls):
        return 'centro_resultado'



from viggocore.common import subsystem
from viggofiscal.subsystem.parametrizacao.ncm_ibpt \
    import resource, controller, router, manager

subsystem = subsystem.Subsystem(resource=resource.NcmIbpt,
                                controller=controller.Controller,
                                manager=manager.Manager,
                                router=router.Router)



from enum import Enum

import sqlalchemy
from sqlalchemy import orm
from viggocore.common.subsystem import entity
from viggocore.database import db


class CONTA_DRE_CLASSIFICACAO(Enum):
    ATIVO = 'ATIVO'
    PASSIVO = 'PASSIVO'
    RECEITA = 'RECEITA'
    DESPESA = 'DESPESA'
    RESULTADO = 'RESULTADO'
    CUSTO = 'CUSTO'


class ContaDre(entity.Entity, db.Model):

    attributes = ['domain_id', 'descricao', 'classificacao', 'codigo_reduzido']
    attributes += entity.Entity.attributes

    domain_id = db.Column(
        db.CHAR(32), db.ForeignKey('domain.id'), nullable=False)

    domain = orm.relationship(
        'Domain', backref=orm.backref('conta_dre_domain'))

    descricao = db.Column(db.String(255), nullable=False)
    classificacao = db.Column(sqlalchemy.Enum(CONTA_DRE_CLASSIFICACAO),
                              nullable=False)
    codigo_reduzido = db.Column(db.String(50), nullable=True)

    def __init__(self, id, domain_id, descricao, classificacao,
                 codigo_reduzido=None,
                 active=True, created_at=None, created_by=None,
                 updated_at=None, updated_by=None, tag=None):
        super().__init__(id, active, created_at, created_by,
                         updated_at, updated_by, tag)
        self.domain_id = domain_id
        self.descricao = descricao
        self.classificacao = classificacao
        self.codigo_reduzido = codigo_reduzido

    @classmethod
    def individual(cls):
        return 'conta_dre'



from sqlalchemy import and_, or_
from viggocore.common import exception, manager
from viggocore.common.subsystem import operation
from viggocore.common.subsystem.pagination import Pagination

from viggofiscal.subsystem.parametrizacao.serial_fiscal.resource import \
    SerieFiscal
from viggofiscal.subsystem.parametrizacao.terminal.resource import Terminal


class Create(operation.Create):

    def pre(self, session, **kwargs):
        serie = int(kwargs.get('serie', '9999'))
        if serie < 0 or serie > 999:
            raise exception.BadRequest(
                'A serie tem que ser um valor entre 0 e 999.')
        return super().pre(session, **kwargs)


class Update(operation.Update):

    def do(self, session, **kwargs):
        kwargs.pop('ambiente', None)
        kwargs.pop('serie', None)
        kwargs.pop('modelo', None)
        return super().do(session=session, **kwargs)


class List(operation.List):

    def do(self, session, **kwargs):
        query = session.query(SerieFiscal)
        query = self.manager.apply_filters(query, SerieFiscal, **kwargs)

        dict_compare = {}
        query = self.manager.apply_filters_includes(
            query, dict_compare, **kwargs)

        total_rows = None
        if self.manager.with_pagination(**kwargs):
            total_rows = query.count()

        pagination = Pagination.get_pagination(SerieFiscal, **kwargs)
        if pagination.order_by is not None:
            pagination.adjust_order_by(SerieFiscal)
        query = self.driver.apply_pagination(query, pagination)
        result = query.all()

        return (result, total_rows)


class GetNextUltimoDoc(operation.Operation):
    def pre(self, session, id, **kwargs):
        serie_fiscal = self.manager.get(id=id)
        if not serie_fiscal:
            raise exception.NotFound('ERROR! serie_fiscal not found')
        self.serie_fiscal_id = serie_fiscal.id

        return True

    def do(self, session, **kwargs):
        next_ultimo_doc = self.driver.get_next_ultimo_doc(
            session, self.serie_fiscal_id)

        if next_ultimo_doc is None:
            raise exception.ViggoCoreException(
                'Não foi possível retornar o próximo ultimo_doc ' +
                'da serie_fiscal')

        return next_ultimo_doc


class GetDisponiveis(operation.List):

    def do(self, session, **kwargs):
        serie_fiscal_id = kwargs.pop('serie_fiscal_id', None)

        query = session.query(SerieFiscal). \
            join(Terminal, Terminal.serie_fiscal_id == SerieFiscal.id,
                 isouter=True). \
            filter(and_(or_(Terminal.serie_fiscal_id == None,  # noqa: E711
                            SerieFiscal.id == serie_fiscal_id),
                        SerieFiscal.modelo == 'NFCE',
                        SerieFiscal.status == 'ATIVO'))
        query = self.manager.apply_filters(query, SerieFiscal, **kwargs)

        dict_compare = {"terminal.": Terminal}
        query = self.manager.apply_filters_includes(
            query, dict_compare, **kwargs)
        query = query.distinct()

        total_rows = None
        if self.manager.with_pagination(**kwargs):
            total_rows = query.count()

        pagination = Pagination.get_pagination(SerieFiscal, **kwargs)
        if pagination.order_by is not None:
            pagination.order_by = 'serie_fiscal.serie'
        query = self.driver.apply_pagination(query, pagination)
        result = query.all()

        return (result, total_rows)


class Manager(manager.CommonManager):

    def __init__(self, driver):
        super(Manager, self).__init__(driver)
        self.create = Create(self)
        self.update = Update(self)
        self.list = List(self)
        self.get_next_ultimo_doc = GetNextUltimoDoc(self)
        self.get_disponiveis = GetDisponiveis(self)



from sqlalchemy import UniqueConstraint, orm

from viggocore.database import db
from viggocore.common.subsystem import entity


class Terminal(entity.Entity, db.Model):

    attributes = ['domain_id', 'serie_fiscal_id', 'portador_id',
                  'descricao']
    attributes += entity.Entity.attributes

    domain_id = db.Column(
        db.CHAR(32), db.ForeignKey('domain.id'), nullable=False)
    serie_fiscal_id = db.Column(
        db.CHAR(32), db.ForeignKey('serie_fiscal.id'), nullable=True)
    portador_id = db.Column(
        db.CHAR(32), db.ForeignKey('portador.id'), nullable=False)
    descricao = db.Column(db.String(80), nullable=False)

    domain = orm.relationship(
        'Domain', backref=orm.backref('terminal_domain'))
    serie_fiscal = orm.relationship(
        'SerieFiscal', backref=orm.backref('terminal_serie_fiscal'))
    portador = orm.relationship(
        'Portador', backref=orm.backref('terminal_portador'),
        viewonly=True)

    operadores = orm.relationship(
        "TerminalOperador",
        backref=orm.backref('terminal_terminal_operaador'),
        cascade='delete,delete-orphan,save-update')

    __table_args__ = (
        UniqueConstraint(
            'domain_id', 'serie_fiscal_id',
            name='terminal_domain_id_serie_fiscal_id_uk'),
        UniqueConstraint(
            'domain_id', 'descricao',
            name='terminal_domain_id_descricao_uk'),)

    def __init__(self, id, domain_id, portador_id,
                 descricao, serie_fiscal_id=None,
                 active=True, created_at=None, created_by=None,
                 updated_at=None, updated_by=None, tag=None):
        super().__init__(id, active, created_at, created_by,
                         updated_at, updated_by, tag)
        self.domain_id = domain_id
        self.serie_fiscal_id = serie_fiscal_id
        self.portador_id = portador_id
        self.descricao = descricao

    def get_user_id_dos_operadores(self):
        user_ids = []
        for operador in self.operadores:
            user_ids.append(operador.user_id)
        return user_ids

    @classmethod
    def individual(cls):
        return 'terminal'

    @classmethod
    def collection(cls):
        return 'terminais'

    @classmethod
    def embedded(cls):
        return ['operadores']


class TerminalOperador(entity.Entity, db.Model):

    attributes = ['id', 'user_id']
    attributes += entity.Entity.attributes

    terminal_id = db.Column(
        db.CHAR(32), db.ForeignKey('terminal.id'), nullable=False)
    user_id = db.Column(
        db.CHAR(32), db.ForeignKey('user.id'), nullable=False)

    user = orm.relationship(
        'User', backref=orm.backref('terminal_operador_user'))

    def __init__(self, id, terminal_id, user_id,
                 active=True, created_at=None, created_by=None,
                 updated_at=None, updated_by=None, tag=None):
        super().__init__(id, active, created_at, created_by,
                         updated_at, updated_by, tag)
        self.terminal_id = terminal_id
        self.user_id = user_id



from sqlalchemy import func, select
from viggocore.common.subsystem import driver


class Driver(driver.Driver):

    def get_next_ultimo_doc(self, session, serie_fiscal_id):
        result = None
        get_next_ultimo_doc_func = func.serie_fiscal_next_ultimo_doc
        statement = select(get_next_ultimo_doc_func(serie_fiscal_id))
        row_tuple = session.execute(statement).first()
        if len(row_tuple) > 0:
            result = row_tuple[0]
        return result



from enum import Enum

import sqlalchemy
from sqlalchemy import orm
from viggocore.common.subsystem import entity
from viggocore.database import db


class PortadorTipo(Enum):
    INTERNO = 'INTERNO'
    EXTERNO = 'EXTERNO'


class Portador(entity.Entity, db.Model):

    attributes = ['domain_id', 'tipo', 'descricao', 'padrao',
                  'codigo_banco', 'nome_banco', 'saldo',
                  'numero_conta', 'digito_conta', 'codigo_agencia',
                  'digito_agencia', 'saldo_inicial', 'ligado_a_terminal']
    attributes += entity.Entity.attributes

    domain_id = db.Column(
        db.CHAR(32), db.ForeignKey('domain.id'), nullable=False)

    domain = orm.relationship(
        'Domain', backref=orm.backref('portador_domain'))

    tipo = db.Column(sqlalchemy.Enum(PortadorTipo), nullable=False)
    descricao = db.Column(db.String(100), nullable=False)
    padrao = db.Column(db.Boolean(), nullable=False)
    codigo_banco = db.Column(db.CHAR(3), nullable=True)
    nome_banco = db.Column(db.String(100), nullable=True)
    numero_conta = db.Column(db.String(20), nullable=True)
    digito_conta = db.Column(db.CHAR(1), nullable=True)
    codigo_agencia = db.Column(db.String(20), nullable=True)
    digito_agencia = db.Column(db.CHAR(1), nullable=True)
    saldo = db.Column(db.Numeric(15, 2), nullable=False, server_default='0.0')
    saldo_inicial = db.Column(db.Numeric(15, 2), nullable=False,
                              server_default='0.0')

    terminais = orm.relationship(
        'Terminal', backref=orm.backref('portador_terminal'),
        viewonly=True)

    def __init__(self, id, domain_id, tipo, descricao, padrao,
                 saldo_inicial=0.0, saldo=0.0, codigo_banco=None,
                 nome_banco=None, numero_conta=None, digito_conta=None,
                 codigo_agencia=None, digito_agencia=None,
                 active=True, created_at=None, created_by=None,
                 updated_at=None, updated_by=None, tag=None):
        super().__init__(id, active, created_at, created_by,
                         updated_at, updated_by, tag)
        self.domain_id = domain_id
        self.tipo = tipo
        self.descricao = descricao
        self.padrao = padrao
        self.codigo_banco = codigo_banco
        self.nome_banco = nome_banco
        self.numero_conta = numero_conta
        self.digito_conta = digito_conta
        self.codigo_agencia = codigo_agencia
        self.digito_agencia = digito_agencia
        self.saldo = saldo
        self.saldo_inicial = saldo_inicial

    @property
    def ligado_a_terminal(self):
        if self.terminais is None or len(self.terminais) == 0:
            return False
        return True

    @classmethod
    def collection(cls):
        return 'portadores'



import json
from enum import Enum

import sqlalchemy
from typing import Any
from sqlalchemy import ForeignKeyConstraint, orm
from viggocore.common.subsystem import entity
from viggocore.common import exception
from viggocore.database import db


class Crt(Enum):
    SIMPLES_NACIONAL = 1
    SN_EXC_REC_BRUTA = 2
    REGIME_NORMAL = 3
    MEI = 4


class RegPisCofins(Enum):
    CUMULATIVO = 1
    NAO_CUMULATIVO = 2


class DomainOrg(entity.Entity, db.Model):

    attributes = ['crt', 'cpf_cnpj', 'insc_est', 'razao_social',
                  'nome_fantasia', 'insc_mun', 'cnae', 'cpf_cnpj_contador',
                  'reg_pis_cofins', 'aliq_pis', 'aliq_cofins', 'cred_sn',
                  'email', 'fone', 'settings']
    attributes += entity.Entity.attributes

    domain = orm.relationship(
        'Domain', backref=orm.backref('domain_org_domain'))

    crt = db.Column(sqlalchemy.Enum(Crt), nullable=False)
    cpf_cnpj = db.Column(db.String(14), nullable=False, unique=True)
    insc_est = db.Column(db.String(14), nullable=False)
    razao_social = db.Column(db.String(100), nullable=False)
    nome_fantasia = db.Column(db.String(100), nullable=False)
    insc_mun = db.Column(db.String(15), nullable=True)
    cnae = db.Column(db.String(15), nullable=True)
    cpf_cnpj_contador = db.Column(db.String(14), nullable=True)
    reg_pis_cofins = db.Column(sqlalchemy.Enum(RegPisCofins), nullable=True)
    aliq_pis = db.Column(db.Numeric(7, 2), nullable=False,
                         default=0, server_default="0.0")
    aliq_cofins = db.Column(db.Numeric(7, 2), nullable=False,
                            default=0, server_default="0.0")
    cred_sn = db.Column(db.Numeric(7, 2), nullable=False,
                        default=0, server_default="0.0")
    email = db.Column(db.String(60), nullable=False)
    fone = db.Column(db.String(60), nullable=True)
    _settings = db.Column('settings', db.Text, nullable=False, default='{}',
                          server_default='{}')

    __table_args__ = (ForeignKeyConstraint(['id'], ['domain.id']),)

    def __init__(self, id, crt, cpf_cnpj, insc_est, razao_social,
                 nome_fantasia, email, insc_mun=None, cnae=None,
                 cpf_cnpj_contador=None, reg_pis_cofins=None, aliq_pis=0.0,
                 aliq_cofins=0.0, cred_sn=0.0, fone=None,
                 active=True, created_at=None, created_by=None,
                 updated_at=None, updated_by=None, tag=None):
        super().__init__(id, active, created_at, created_by,
                         updated_at, updated_by, tag)
        self.crt = crt
        self.cpf_cnpj = cpf_cnpj
        self.insc_est = insc_est
        self.razao_social = razao_social
        self.nome_fantasia = nome_fantasia
        self.insc_mun = insc_mun
        self.cnae = cnae
        self.cpf_cnpj_contador = cpf_cnpj_contador
        self.reg_pis_cofins = reg_pis_cofins
        self.aliq_pis = float(aliq_pis)
        self.aliq_cofins = float(aliq_cofins)
        self.cred_sn = float(cred_sn)
        self.email = email
        self.fone = fone

    def _has_setting(self, key: str) -> bool:
        return self.settings.get(key) is not None

    def remove_setting(self, key: str):
        if not self._has_setting(key):
            raise exception.BadRequest(f"Erro! Setting {key} not exists")

        settings = self.settings
        value = settings.pop(key)
        self._save_settings(settings)

        return value

    def update_setting(self, key: str, value: Any):
        settings = self.settings
        settings[key] = value
        self._save_settings(settings)
        return value

    @property
    def settings(self):
        try:
            settings_str = '{}' if self._settings is None else self._settings
            return json.loads(settings_str)
        except Exception:
            return {}

    def _save_settings(self, settings: dict):
        self._settings = json.dumps(settings, default=str)

    def get_api_fiscal_in_settings(self):
        return self.settings.get('api_fiscal', None)

    @classmethod
    def individual(cls):
        return 'domain_org'



from viggocore.database import db
from viggocore.common.subsystem import entity


class Cfop(entity.Entity, db.Model):

    attributes = ['cfop', 'descricao', 'observacao', 'tipo']
    attributes += entity.Entity.attributes

    cfop = db.Column(db.String(5), nullable=False, unique=True)
    descricao = db.Column(db.String(200), nullable=False)
    observacao = db.Column(db.String(800), nullable=False)
    tipo = db.Column(db.Numeric(1), nullable=False)

    def __init__(self, id, cfop, descricao, observacao, tipo,
                 active=True, created_at=None, created_by=None,
                 updated_at=None, updated_by=None, tag=None):
        super().__init__(id, active, created_at, created_by,
                         updated_at, updated_by, tag)
        self.cfop = cfop
        self.descricao = descricao
        self.observacao = observacao
        self.tipo = tipo



import flask
from viggocore.common import exception
from viggocore.common.subsystem import manager


class Manager(manager.Manager):

    def __init__(self, driver):
        super(Manager, self).__init__(driver)

    def get_erro_existe(self, **kwargs):
        try:
            erros = self.list(**kwargs)
        except exception.ViggoCoreException as exc:
            return flask.Response(response=exc.message,
                                  status=exc.status)

        if len(erros) > 0:
            response = erros[0]
        else:
            response = None

        return response
